package com.hh.lib_device.deviceui;

import android.app.Activity;

/**
 * 其余设备
 * 空实现
 */
public class NullDeviceUI implements IDeviceUI {

    //用volatile关键字确保 instance 在多线程下的可见性
    private static volatile NullDeviceUI instance = null;

    //将构造方法私有化，禁止外部通过构造方法创建实例
    private NullDeviceUI() {}

    //提供一个公共的访问方法，用于获取该类的唯一实例
    public static NullDeviceUI getInstance() {
        //如果instance为空，就进行实例化
        if (instance == null) {
            //保证多线程下只有一个线程进行实例化
            synchronized (NullDeviceUI.class) {
                //第二次判断，避免多线程下创建多个实例
                if (instance == null) {
                    instance = new NullDeviceUI();
                }
            }
        }
        return instance;
    }


    @Override
    public void showNotification(Activity activity) {
    }

    @Override
    public void hideNotification(Activity activity) {
    }

    @Override
    public void showNavigationBar(Activity activity) {
    }

    @Override
    public void hideNavigationBar(Activity activity) {
    }

    @Override
    public void showFillLight(Activity activity) {

    }

    @Override
    public void hideFillLight(Activity activity) {

    }

    @Override
    public void reboot() {

    }

    @Override
    public void onRelease() {
        instance = null;
    }
}
