package com.hh.lib_serialport;


import android.text.TextUtils;
import android.util.Log;

import com.google.gson.Gson;
import com.hh.lib_base.BuildConfig;
import com.hh.lib_base.log.AppLogUtils;
import com.hh.lib_serialport.bean.ComBean;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.InvalidParameterException;

import android_serialport_api.SerialPort;

/**
 * @author benjaminwan
 * 串口辅助工具类
 */
public abstract class SerialHelper {


    // 当前类名
    public String mTag = "HHArome--"+ getClass().getSimpleName() + "--";

    private SerialPort mSerialPort;
    private OutputStream mOutputStream;
    private InputStream mInputStream;
    private ReadThread mReadThread;
    private SendThread mSendThread;
    private String sPort = "/dev/s3c2410_serial0";
    private int iBaudRate = 9600;
    private boolean _isOpen = false;
    private byte[] _bLoopData = new byte[]{0x30};
    private int iDelay = 500;

    //默认 0-8-1
    private char parity = '0';//char校验类型 取值N ,E, O,,S
    private int dataBits = 8;//dataBits 类型 int数据位 取值 位7或8
    private int stopBits = 1;//stopBits 类型 int 停止位 取值1 或者 2

    /**
     * 设定校验位等
     */
    public void setN81(String idataBits, String istopBits, String cparity) {
        this.parity = cparity.charAt(0);
        this.dataBits = Integer.parseInt(idataBits);
        this.stopBits = Integer.parseInt(istopBits);
    }




    //----------------------------------------------------
    public SerialHelper(String sPort, int iBaudRate) {
        this.sPort = sPort;
        this.iBaudRate = iBaudRate;
    }

    public SerialHelper() {
        this("/dev/s3c2410_serial0", 9600);
    }

    public SerialHelper(String sPort) {
        this(sPort, 9600);
    }

    public SerialHelper(String sPort, String sBaudRate) {
        this(sPort, Integer.parseInt(sBaudRate));
    }

    //----------------------------------------------------
    public void open() throws SecurityException, IOException, InvalidParameterException {
        if(BuildConfig.LOG_DEBUG) {
            Log.e(mTag, "串口地址" + sPort + " 波特率:" + iBaudRate + " 数据位:" + dataBits + " 停止位:" + stopBits + " 校验位:" + parity);
        }
       if(TextUtils.isEmpty(sPort)){
           return;
       }


        File device = new File(sPort);
   /*     mSerialPort= SerialPort
                .newBuilder(device,iBaudRate)
                .dataBits(dataBits)
                .stopBits(stopBits)
                .parity(parity)
                .flags(0)
                .build();*/
        mSerialPort = new SerialPort(device,iBaudRate,0);

        mOutputStream = mSerialPort.getOutputStream();
        mInputStream = mSerialPort.getInputStream();
        mReadThread = new ReadThread();
        mReadThread.start();
//        mSendThread = new SendThread();
//        mSendThread.setSuspendFlag();
//        mSendThread.start();
        _isOpen = true;
    }

    //----------------------------------------------------
    public void close() {
        if (mReadThread != null)
            mReadThread.interrupt();
        if (mSerialPort != null) {
            try {
                mSerialPort.close();
            }catch (Exception e){

            }
            mSerialPort = null;
        }
        _isOpen = false;
    }

    //----------------------------------------------------
    public void send(byte[] bOutArray) {
        if (bOutArray == null) {
            AppLogUtils.e("-----------数据发送异常：bOutArray=" + bOutArray);
            return;
        }
        if(mOutputStream == null){
            AppLogUtils.e("-----------数据发送异常：mOutputStream=" + mOutputStream);
            return;
        }
        try {
            mOutputStream.write(bOutArray);
            mOutputStream.flush(); // 清空缓冲区，确保数据发送出去
           // Log.i(mTag, "-----------数据发送-转十六进制："+MyFunc.ByteArrToHex(bOutArray));
          //  Log.i(mTag, "-----------数据发送-转String："+MyFunc.ByteArrToStr(bOutArray));
        } catch (IOException e) {
            e.printStackTrace();
            Log.e(mTag, "-----------数据发送异常："+e.toString());
        }
    }


    //----------------------------------------------------
    public void sendHex(String sHex) {
        byte[] bOutArray = MyFunc.HexToByteArr(sHex);
        send(bOutArray);
    }

    //----------------------------------------------------
    public void sendTxt(String sTxt) {
        //   byte[] bOutArray = sTxt.getBytes();
        byte[] bytes = HexCommandtoByte(sTxt.getBytes());//字符转成byte数组
        Log.e(mTag, new Gson().toJson(bytes));
        send(bytes);
    }

    //  十六进制的字符串转换成byte数组
    public static byte[] HexCommandtoByte(byte[] data) {
        if (data == null) {
            return null;
        }
        int nLength = data.length;

        String strTemString = new String(data, 0, nLength);
        String[] strings = strTemString.split(" ");
        nLength = strings.length;
        data = new byte[nLength];
        for (int i = 0; i < nLength; i++) {
            if (strings[i].length() != 2) {
                data[i] = 00;
                continue;
            }
            try {
                data[i] = (byte) Integer.parseInt(strings[i], 16);
            } catch (Exception e) {
                data[i] = 00;
                continue;
            }
        }

        return data;
    }

    //----------------------------------------------------
    private class ReadThread extends Thread {
        @Override
        public void run() {
            super.run();
            while (!isInterrupted()) {
                try {
                    Thread.sleep(200);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                try {
                    byte[] buffer = new byte[512];
                    int size = mInputStream.read(buffer);
                    if (size > 0) {
                      //  Log.e(mTag, "-----------数据处理 size   " + size + "");
                     //   Log.e(mTag, "-----------数据处理 bRec  " + new Gson().toJson(buffer));
                        ComBean ComRecData = new ComBean(sPort, buffer, size);
                        onDataReceived(ComRecData);//添加QueueList内容
                    } else {
                        //当数据为空或传输数据结束后
                        DoQueue();
                    }
                } catch (Throwable e) {
                    e.printStackTrace();
                    return;
                }
            }
        }
    }

    //----------------------------------------------------
    private class SendThread extends Thread {
        public boolean suspendFlag = true;// 控制线程的执行

        @Override
        public void run() {
            super.run();
            while (!isInterrupted()) {
                synchronized (this) {
                    while (suspendFlag) {
                        try {
                            wait();
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
                }
              //  Log.e(mTag,"getbLoopData--"+getbLoopData());
                send(getbLoopData());
                try {
                    Thread.sleep(iDelay);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

        //线程暂停
        public void setSuspendFlag() {
            this.suspendFlag = true;
        }

        //唤醒线程
        public synchronized void setResume() {
            this.suspendFlag = false;
            notify();
        }
    }

    //----------------------------------------------------
    public int getBaudRate() {
        return iBaudRate;
    }

    public boolean setBaudRate(int iBaud) {
        if (_isOpen) {
            return false;
        } else {
            iBaudRate = iBaud;
            return true;
        }
    }

    public boolean setBaudRate(String sBaud) {
        int iBaud = Integer.parseInt(sBaud);
        return setBaudRate(iBaud);
    }

    //----------------------------------------------------
    public String getPort() {
        return sPort;
    }

    public boolean setPort(String sPort) {
        if (_isOpen) {
            return false;
        } else {
            this.sPort = sPort;
            return true;
        }
    }

    //----------------------------------------------------
    public boolean isOpen() {
        return _isOpen;
    }

    //----------------------------------------------------
    public byte[] getbLoopData() {
        return _bLoopData;
    }

    //----------------------------------------------------
    public void setbLoopData(byte[] bLoopData) {
        this._bLoopData = bLoopData;
    }

    //----------------------------------------------------
    public void setTxtLoopData(String sTxt) {
        this._bLoopData = sTxt.getBytes();
    }

    //----------------------------------------------------
    public void setHexLoopData(String sHex) {
        this._bLoopData = MyFunc.HexToByteArr(sHex);
    }

    //----------------------------------------------------
    public int getiDelay() {
        return iDelay;
    }

    //----------------------------------------------------
    public void setiDelay(int iDelay) {
        this.iDelay = iDelay;
    }

    //----------------------------------------------------
    public void startSend() {
        if (mSendThread != null) {
            mSendThread.setResume();
        }
    }

    //----------------------------------------------------
    public void stopSend() {
        if (mSendThread != null) {
            mSendThread.setSuspendFlag();
        }
    }

    //----------------------------------------------------
    protected abstract void onDataReceived(ComBean ComRecData);

    protected abstract void DoQueue();
}